import { getFirebaseAdminTokenProvider } from './credential';
import { AuthError, AuthErrorCode } from './error.js';
import { emulatorHost, useEmulator } from './firebase.js';
import { formatString } from './utils.js';
import { isEmail, isNonNullObject } from './validator.js';
export class ApiSettings {
    constructor(endpoint, httpMethod = 'POST') {
        this.endpoint = endpoint;
        this.httpMethod = httpMethod;
    }
    getEndpoint() {
        return this.endpoint;
    }
    getHttpMethod() {
        return this.httpMethod;
    }
}
export function getSdkVersion() {
    return '11.2.0';
}
const FIREBASE_AUTH_HEADER = {
    'X-Client-Version': `Node/Admin/${getSdkVersion()}`,
    Accept: 'application/json',
    'Content-Type': 'application/json'
};
const FIREBASE_AUTH_BASE_URL_FORMAT = 'https://identitytoolkit.googleapis.com/{version}/projects/{projectId}{api}';
const FIREBASE_AUTH_EMULATOR_BASE_URL_FORMAT = 'http://{host}/identitytoolkit.googleapis.com/{version}/projects/{projectId}{api}';
class AuthResourceUrlBuilder {
    constructor(version = 'v1', credential) {
        this.version = version;
        this.credential = credential;
        if (useEmulator()) {
            this.urlFormat = formatString(FIREBASE_AUTH_EMULATOR_BASE_URL_FORMAT, {
                host: emulatorHost()
            });
        }
        else {
            this.urlFormat = FIREBASE_AUTH_BASE_URL_FORMAT;
        }
    }
    async getUrl(api, params) {
        const baseParams = {
            version: this.version,
            projectId: await this.credential.getProjectId(),
            api: api || ''
        };
        const baseUrl = formatString(this.urlFormat, baseParams);
        return formatString(baseUrl, params || {});
    }
}
export const FIREBASE_AUTH_CREATE_SESSION_COOKIE = new ApiSettings(':createSessionCookie', 'POST');
export const FIREBASE_AUTH_GET_ACCOUNT_INFO = new ApiSettings('/accounts:lookup', 'POST');
export const FIREBASE_AUTH_DELETE_ACCOUNT = new ApiSettings('/accounts:delete', 'POST');
export const FIREBASE_AUTH_SET_ACCOUNT_INFO = new ApiSettings('/accounts:update', 'POST');
export const FIREBASE_AUTH_SIGN_UP_NEW_USER = new ApiSettings('/accounts', 'POST');
export const FIREBASE_AUTH_LIST_USERS_INFO = new ApiSettings('/accounts:batchGet', 'GET');
export class AbstractAuthRequestHandler {
    static getErrorCode(response) {
        return ((isNonNullObject(response) &&
            response.error &&
            response.error.message) ||
            null);
    }
    constructor(credential, options = {}) {
        this.options = options;
        this.getToken = useEmulator()
            ? () => Promise.resolve({
                accessToken: 'owner',
                expirationTime: Infinity
            })
            : getFirebaseAdminTokenProvider(credential).getToken;
    }
    prepareRequest(request) {
        if (!this.options.tenantId) {
            return request;
        }
        return {
            ...request,
            tenantId: this.options.tenantId
        };
    }
    getAccountInfoByUid(uid) {
        const request = {
            localId: [uid]
        };
        return this.invokeRequestHandler(this.getAuthUrlBuilder(), FIREBASE_AUTH_GET_ACCOUNT_INFO, request);
    }
    deleteAccount(uid) {
        return this.invokeRequestHandler(this.getAuthUrlBuilder(), FIREBASE_AUTH_DELETE_ACCOUNT, {
            localId: uid
        });
    }
    getAccountInfoByEmail(email) {
        if (!isEmail(email)) {
            return Promise.reject(new AuthError(AuthErrorCode.INVALID_ARGUMENT, 'Invalid e-mail address'));
        }
        const request = {
            email: [email]
        };
        return this.invokeRequestHandler(this.getAuthUrlBuilder(), FIREBASE_AUTH_GET_ACCOUNT_INFO, request);
    }
    createNewAccount(properties) {
        const request = {
            ...properties
        };
        if (typeof request.photoURL !== 'undefined') {
            request.photoUrl = request.photoURL;
            delete request.photoURL;
        }
        if (typeof request.uid !== 'undefined') {
            request.localId = request.uid;
            delete request.uid;
        }
        if (request.multiFactor) {
            if (Array.isArray(request.multiFactor.enrolledFactors) &&
                request.multiFactor.enrolledFactors.length > 0) {
                const mfaInfo = [];
                try {
                    request.multiFactor.enrolledFactors.forEach((multiFactorInfo) => {
                        if ('enrollmentTime' in multiFactorInfo) {
                            throw new AuthError(AuthErrorCode.INVALID_ARGUMENT, '"enrollmentTime" is not supported when adding second factors via "createUser()"');
                        }
                        else if ('uid' in multiFactorInfo) {
                            throw new AuthError(AuthErrorCode.INVALID_ARGUMENT, '"uid" is not supported when adding second factors via "createUser()"');
                        }
                        mfaInfo.push(convertMultiFactorInfoToServerFormat(multiFactorInfo));
                    });
                }
                catch (e) {
                    return Promise.reject(e);
                }
                request.mfaInfo = mfaInfo;
            }
            delete request.multiFactor;
        }
        return this.invokeRequestHandler(this.getAuthUrlBuilder(), FIREBASE_AUTH_SIGN_UP_NEW_USER, request).then((response) => {
            return response.localId;
        });
    }
    createSessionCookie(idToken, expiresInMs) {
        const request = {
            idToken,
            validDuration: expiresInMs / 1000
        };
        return this.invokeRequestHandler(this.getAuthUrlBuilder(), FIREBASE_AUTH_CREATE_SESSION_COOKIE, request).then((response) => response.sessionCookie);
    }
    updateExistingAccount(uid, properties) {
        const request = {
            ...properties,
            deleteAttribute: [],
            localId: uid
        };
        const deletableParams = {
            displayName: 'DISPLAY_NAME',
            photoURL: 'PHOTO_URL'
        };
        request.deleteAttribute = [];
        for (const key in deletableParams) {
            if (request[key] === null) {
                request.deleteAttribute.push(deletableParams[key]);
                delete request[key];
            }
        }
        if (request.deleteAttribute.length === 0) {
            delete request.deleteAttribute;
        }
        if (request.phoneNumber === null) {
            if (request.deleteProvider) {
                request.deleteProvider.push('phone');
            }
            else {
                request.deleteProvider = ['phone'];
            }
            delete request.phoneNumber;
        }
        if (typeof request.providerToLink !== 'undefined') {
            request.linkProviderUserInfo = { ...request.providerToLink };
            delete request.providerToLink;
            request.linkProviderUserInfo.rawId = request.linkProviderUserInfo.uid;
            delete request.linkProviderUserInfo.uid;
        }
        if (typeof request.providersToUnlink !== 'undefined') {
            if (!Array.isArray(request.deleteProvider)) {
                request.deleteProvider = [];
            }
            request.deleteProvider = request.deleteProvider.concat(request.providersToUnlink);
            delete request.providersToUnlink;
        }
        if (typeof request.photoURL !== 'undefined') {
            request.photoUrl = request.photoURL;
            delete request.photoURL;
        }
        if (typeof request.disabled !== 'undefined') {
            request.disableUser = request.disabled;
            delete request.disabled;
        }
        if (request.multiFactor) {
            if (request.multiFactor.enrolledFactors === null) {
                request.mfa = {};
            }
            else if (Array.isArray(request.multiFactor.enrolledFactors)) {
                request.mfa = {
                    enrollments: []
                };
                try {
                    request.multiFactor.enrolledFactors.forEach((multiFactorInfo) => {
                        request.mfa.enrollments.push(convertMultiFactorInfoToServerFormat(multiFactorInfo));
                    });
                }
                catch (e) {
                    return Promise.reject(e);
                }
                if (request.mfa.enrollments.length === 0) {
                    delete request.mfa.enrollments;
                }
            }
            delete request.multiFactor;
        }
        return this.invokeRequestHandler(this.getAuthUrlBuilder(), FIREBASE_AUTH_SET_ACCOUNT_INFO, request).then((response) => {
            return response.localId;
        });
    }
    setCustomUserClaims(uid, customUserClaims) {
        if (customUserClaims === null) {
            customUserClaims = {};
        }
        const request = {
            localId: uid,
            customAttributes: JSON.stringify(customUserClaims)
        };
        return this.invokeRequestHandler(this.getAuthUrlBuilder(), FIREBASE_AUTH_SET_ACCOUNT_INFO, request).then((response) => {
            return response.localId;
        });
    }
    listUsers(nextPageToken, maxResults) {
        const request = {
            nextPageToken,
            maxResults
        };
        return this.invokeRequestHandler(this.getAuthUrlBuilder(), FIREBASE_AUTH_LIST_USERS_INFO, request);
    }
    getSearchParams(requestData) {
        const searchParams = new URLSearchParams();
        for (const key in requestData) {
            if (!requestData[key]) {
                continue;
            }
            searchParams.append(key, requestData[key]);
        }
        return searchParams;
    }
    decorateUrlWithParams(url, requestData) {
        const params = this.getSearchParams(requestData);
        const paramsString = params.toString();
        if (!paramsString) {
            return url;
        }
        return `${url}?${paramsString}`;
    }
    async invokeRequestHandler(urlBuilder, apiSettings, requestData, additionalResourceParams) {
        let url = await urlBuilder.getUrl(apiSettings.getEndpoint(), additionalResourceParams);
        const token = await this.getToken();
        const init = {
            method: apiSettings.getHttpMethod(),
            headers: {
                ...FIREBASE_AUTH_HEADER,
                Authorization: `Bearer ${token.accessToken}`
            }
        };
        if (requestData && !['GET', 'HEAD'].includes(apiSettings.getHttpMethod())) {
            init.body = JSON.stringify(this.prepareRequest(requestData));
        }
        if (requestData && ['GET', 'HEAD'].includes(apiSettings.getHttpMethod())) {
            url = this.decorateUrlWithParams(url, this.prepareRequest(requestData));
        }
        const res = await fetch(url, init);
        if (!res.ok) {
            const error = await res.json();
            const errorCode = AbstractAuthRequestHandler.getErrorCode(error);
            if (!errorCode) {
                throw new AuthError(AuthErrorCode.INTERNAL_ERROR, `Error returned from server: ${JSON.stringify(error)}.`);
            }
            throw new AuthError(AuthErrorCode.INTERNAL_ERROR, `Error returned from server: ${JSON.stringify(error)}. Code: ${errorCode}`);
        }
        return await res.json();
    }
    getAuthUrlBuilder() {
        if (!this.authUrlBuilder) {
            this.authUrlBuilder = this.newAuthUrlBuilder();
        }
        return this.authUrlBuilder;
    }
}
export class AuthRequestHandler extends AbstractAuthRequestHandler {
    constructor(credential, options) {
        super(credential, options);
        this.credential = credential;
        this.authResourceUrlBuilder = new AuthResourceUrlBuilder('v2', credential);
    }
    newAuthUrlBuilder() {
        return new AuthResourceUrlBuilder('v1', this.credential);
    }
}
function isPhoneFactor(multiFactorInfo) {
    return multiFactorInfo.factorId === 'phone';
}
function isUTCDateString(dateString) {
    try {
        return (Boolean(dateString) && new Date(dateString).toUTCString() === dateString);
    }
    catch {
        return false;
    }
}
export function convertMultiFactorInfoToServerFormat(multiFactorInfo) {
    let enrolledAt;
    if (typeof multiFactorInfo.enrollmentTime !== 'undefined') {
        if (isUTCDateString(multiFactorInfo.enrollmentTime)) {
            enrolledAt = new Date(multiFactorInfo.enrollmentTime).toISOString();
        }
        else {
            throw new AuthError(AuthErrorCode.INVALID_ARGUMENT, `The second factor "enrollmentTime" for "${multiFactorInfo.uid}" must be a valid ` +
                'UTC date string.');
        }
    }
    if (isPhoneFactor(multiFactorInfo)) {
        const authFactorInfo = {
            mfaEnrollmentId: multiFactorInfo.uid,
            displayName: multiFactorInfo.displayName,
            phoneInfo: multiFactorInfo.phoneNumber,
            enrolledAt
        };
        for (const objKey in authFactorInfo) {
            if (typeof authFactorInfo[objKey] === 'undefined') {
                delete authFactorInfo[objKey];
            }
        }
        return authFactorInfo;
    }
    else {
        throw new AuthError(AuthErrorCode.INVALID_ARGUMENT, `Unsupported second factor "${JSON.stringify(multiFactorInfo)}" provided.`);
    }
}
