import { createRemoteJWKSet, cryptoRuntime, decodeProtectedHeader, errors } from 'jose';
import { debug } from '../debug/index.js';
import { AuthError, AuthErrorCode } from './error.js';
import { useEmulator } from './firebase.js';
import { getPublicCryptoKey, verify } from './jwt/verify.js';
import { isNonNullObject, isURL } from './validator.js';
function getExpiresAt(res) {
    if (!res.headers.has('cache-control')) {
        return 0;
    }
    const cacheControlHeader = res.headers.get('cache-control');
    const parts = cacheControlHeader.split(',');
    const maxAge = parts.reduce((acc, part) => {
        const subParts = part.trim().split('=');
        if (subParts[0] === 'max-age') {
            return +subParts[1];
        }
        return acc;
    }, 0);
    return Date.now() + maxAge * 1000;
}
const keyResponseCache = new Map();
export class UrlKeyFetcher {
    constructor(clientCertUrl) {
        this.clientCertUrl = clientCertUrl;
        if (!isURL(clientCertUrl)) {
            throw new Error('The provided public client certificate URL is not a valid URL.');
        }
    }
    async fetchPublicKeysResponse(url) {
        const res = await fetch(url);
        const headers = {};
        res.headers.forEach((value, key) => {
            headers[key] = value;
        });
        debug('Public keys fetched', {
            responseHeaders: headers,
            cryptoRuntime
        });
        if (!res.ok) {
            let errorMessage = 'Error fetching public keys for Google certs: ';
            const data = await res.json();
            if (data.error) {
                errorMessage += `${data.error}`;
                if (data.error_description) {
                    errorMessage += ' (' + data.error_description + ')';
                }
            }
            else {
                errorMessage += `${await res.text()}`;
            }
            throw new Error(errorMessage);
        }
        const data = await res.json();
        if (data.error) {
            throw new Error('Error fetching public keys for Google certs: ' + data.error);
        }
        const expiresAt = getExpiresAt(res);
        return {
            keys: data,
            expiresAt
        };
    }
    async fetchAndCachePublicKeys(url) {
        debug('No public keys found in cache. Fetching public keys from Google...', {
            cryptoRuntime
        });
        const response = await this.fetchPublicKeysResponse(url);
        keyResponseCache.set(url.toString(), response);
        debug('Public keys cached', {
            cacheKey: url.toString(),
            expiresAt: response.expiresAt,
            cryptoRuntime
        });
        return response.keys;
    }
    async fetchPublicKeys() {
        const url = new URL(this.clientCertUrl);
        const cachedResponse = keyResponseCache.get(url.toString());
        if (!cachedResponse) {
            return this.fetchAndCachePublicKeys(url);
        }
        const { keys, expiresAt } = cachedResponse;
        const now = Date.now();
        debug('Get public keys from cache', {
            expiresAt,
            now,
            cryptoRuntime
        });
        if (expiresAt <= now) {
            return this.fetchAndCachePublicKeys(url);
        }
        return keys;
    }
}
export class JWKSSignatureVerifier {
    constructor(jwksUrl, options) {
        this.options = options;
        if (!isURL(jwksUrl)) {
            throw new Error('The provided JWKS URL is not a valid URL.');
        }
        this.jwksUrl = new URL(jwksUrl);
    }
    async getPublicKey(header) {
        const getKey = createRemoteJWKSet(this.jwksUrl, this.options);
        return getKey(header);
    }
    async verify(token, options) {
        const header = decodeProtectedHeader(token);
        try {
            await verify(token, () => this.getPublicKey(header), options);
        }
        catch (e) {
            if (e instanceof errors.JWKSMultipleMatchingKeys) {
                for await (const publicKey of e) {
                    try {
                        await verify(token, () => Promise.resolve(publicKey), options);
                        return;
                    }
                    catch (innerError) {
                        if (innerError instanceof errors.JWSSignatureVerificationFailed) {
                            continue;
                        }
                        throw innerError;
                    }
                }
                throw new errors.JWSSignatureVerificationFailed();
            }
            throw e;
        }
    }
}
export class PublicKeySignatureVerifier {
    constructor(keyFetcher) {
        this.keyFetcher = keyFetcher;
        if (!isNonNullObject(keyFetcher)) {
            throw new Error('The provided key fetcher is not an object or null.');
        }
    }
    static withCertificateUrl(clientCertUrl) {
        return new PublicKeySignatureVerifier(new UrlKeyFetcher(clientCertUrl));
    }
    async getPublicKey(header) {
        if (useEmulator()) {
            return { type: 'none' };
        }
        return fetchPublicKey(this.keyFetcher, header).then(getPublicCryptoKey);
    }
    async verify(token, options) {
        const header = decodeProtectedHeader(token);
        try {
            await verify(token, () => this.getPublicKey(header), options);
        }
        catch (e) {
            if (e instanceof AuthError && e.code === AuthErrorCode.NO_KID_IN_HEADER) {
                await this.verifyWithoutKid(token, options);
                return;
            }
            throw e;
        }
    }
    async verifyWithoutKid(token, options) {
        const publicKeys = await this.keyFetcher.fetchPublicKeys();
        return this.verifyWithAllKeys(token, publicKeys, options);
    }
    async verifyWithAllKeys(token, keys, options) {
        const promises = [];
        Object.values(keys).forEach((key) => {
            const promise = verify(token, async () => getPublicCryptoKey(key), options)
                .then(() => true)
                .catch((error) => {
                if (error instanceof errors.JWTExpired) {
                    throw error;
                }
                return false;
            });
            promises.push(promise);
        });
        return Promise.all(promises).then((result) => {
            if (result.every((r) => r === false)) {
                throw new AuthError(AuthErrorCode.INVALID_SIGNATURE);
            }
        });
    }
}
export async function fetchPublicKey(fetcher, header) {
    if (!header.kid) {
        throw new AuthError(AuthErrorCode.NO_KID_IN_HEADER);
    }
    const kid = header.kid;
    const publicKeys = await fetcher.fetchPublicKeys();
    if (!Object.prototype.hasOwnProperty.call(publicKeys, kid)) {
        throw new AuthError(AuthErrorCode.NO_MATCHING_KID);
    }
    return publicKeys[kid];
}
