import { addReadonlyGetter, deepCopy } from './utils.js';
import { isNonNullObject } from './validator.js';
import { base64url } from 'jose';
import { AuthError, AuthErrorCode } from './error.js';
const B64_REDACTED = base64url.encode('REDACTED');
function parseDate(time) {
    try {
        const date = new Date(parseInt(time, 10));
        if (!isNaN(date.getTime())) {
            return date.toUTCString();
        }
    }
    catch {
        return null;
    }
    return null;
}
var MultiFactorId;
(function (MultiFactorId) {
    MultiFactorId["Phone"] = "phone";
})(MultiFactorId || (MultiFactorId = {}));
export class MultiFactorInfo {
    static initMultiFactorInfo(response) {
        let multiFactorInfo = null;
        try {
            multiFactorInfo = new PhoneMultiFactorInfo(response);
        }
        catch {
            return null;
        }
        return multiFactorInfo;
    }
    constructor(response) {
        this.initFromServerResponse(response);
    }
    toJSON() {
        return {
            uid: this.uid,
            displayName: this.displayName,
            factorId: this.factorId,
            enrollmentTime: this.enrollmentTime
        };
    }
    initFromServerResponse(response) {
        const factorId = response && this.getFactorId(response);
        if (!factorId || !response || !response.mfaEnrollmentId) {
            throw new AuthError(AuthErrorCode.INTERNAL_ERROR, 'INTERNAL ASSERT FAILED: Invalid multi-factor info response');
        }
        addReadonlyGetter(this, 'uid', response.mfaEnrollmentId);
        addReadonlyGetter(this, 'factorId', factorId);
        addReadonlyGetter(this, 'displayName', response.displayName);
        if (response.enrolledAt) {
            addReadonlyGetter(this, 'enrollmentTime', new Date(response.enrolledAt).toUTCString());
        }
        else {
            addReadonlyGetter(this, 'enrollmentTime', null);
        }
    }
}
export class PhoneMultiFactorInfo extends MultiFactorInfo {
    constructor(response) {
        super(response);
        addReadonlyGetter(this, 'phoneNumber', response.phoneInfo);
    }
    toJSON() {
        return Object.assign(super.toJSON(), {
            phoneNumber: this.phoneNumber
        });
    }
    getFactorId(response) {
        return response && response.phoneInfo ? MultiFactorId.Phone : null;
    }
}
export class MultiFactorSettings {
    constructor(response) {
        const parsedEnrolledFactors = [];
        if (!isNonNullObject(response)) {
            throw new AuthError(AuthErrorCode.INTERNAL_ERROR, 'INTERNAL ASSERT FAILED: Invalid multi-factor response');
        }
        else if (response.mfaInfo) {
            response.mfaInfo.forEach((factorResponse) => {
                const multiFactorInfo = MultiFactorInfo.initMultiFactorInfo(factorResponse);
                if (multiFactorInfo) {
                    parsedEnrolledFactors.push(multiFactorInfo);
                }
            });
        }
        addReadonlyGetter(this, 'enrolledFactors', Object.freeze(parsedEnrolledFactors));
    }
    toJSON() {
        return {
            enrolledFactors: this.enrolledFactors.map((info) => info.toJSON())
        };
    }
}
export class UserMetadata {
    constructor(response) {
        addReadonlyGetter(this, 'creationTime', parseDate(response.createdAt));
        addReadonlyGetter(this, 'lastSignInTime', parseDate(response.lastLoginAt));
        const lastRefreshAt = response.lastRefreshAt
            ? new Date(response.lastRefreshAt).toUTCString()
            : null;
        addReadonlyGetter(this, 'lastRefreshTime', lastRefreshAt);
    }
    toJSON() {
        return {
            lastSignInTime: this.lastSignInTime,
            creationTime: this.creationTime,
            lastRefreshTime: this.lastRefreshTime
        };
    }
}
export class UserInfo {
    constructor(response) {
        if (!response.rawId || !response.providerId) {
            throw new AuthError(AuthErrorCode.INTERNAL_ERROR, 'INTERNAL ASSERT FAILED: Invalid user info response');
        }
        addReadonlyGetter(this, 'uid', response.rawId);
        addReadonlyGetter(this, 'displayName', response.displayName);
        addReadonlyGetter(this, 'email', response.email);
        addReadonlyGetter(this, 'photoURL', response.photoUrl);
        addReadonlyGetter(this, 'providerId', response.providerId);
        addReadonlyGetter(this, 'phoneNumber', response.phoneNumber);
    }
    toJSON() {
        return {
            uid: this.uid,
            displayName: this.displayName,
            email: this.email,
            photoURL: this.photoURL,
            providerId: this.providerId,
            phoneNumber: this.phoneNumber
        };
    }
}
export class UserRecord {
    constructor(response) {
        if (!response.localId) {
            throw new AuthError(AuthErrorCode.INTERNAL_ERROR, 'INTERNAL ASSERT FAILED: Invalid user response');
        }
        addReadonlyGetter(this, 'uid', response.localId);
        addReadonlyGetter(this, 'email', response.email);
        addReadonlyGetter(this, 'emailVerified', !!response.emailVerified);
        addReadonlyGetter(this, 'displayName', response.displayName);
        addReadonlyGetter(this, 'photoURL', response.photoUrl);
        addReadonlyGetter(this, 'phoneNumber', response.phoneNumber);
        addReadonlyGetter(this, 'disabled', response.disabled || false);
        addReadonlyGetter(this, 'metadata', new UserMetadata(response));
        const providerData = [];
        for (const entry of response.providerUserInfo || []) {
            providerData.push(new UserInfo(entry));
        }
        addReadonlyGetter(this, 'providerData', providerData);
        if (response.passwordHash === B64_REDACTED) {
            addReadonlyGetter(this, 'passwordHash', undefined);
        }
        else {
            addReadonlyGetter(this, 'passwordHash', response.passwordHash);
        }
        addReadonlyGetter(this, 'passwordSalt', response.salt);
        if (response.customAttributes) {
            addReadonlyGetter(this, 'customClaims', JSON.parse(response.customAttributes));
        }
        let validAfterTime = null;
        if (typeof response.validSince !== 'undefined') {
            validAfterTime = parseDate(parseInt(response.validSince, 10) * 1000);
        }
        addReadonlyGetter(this, 'tokensValidAfterTime', validAfterTime || undefined);
        addReadonlyGetter(this, 'tenantId', response.tenantId);
        const multiFactor = new MultiFactorSettings(response);
        if (multiFactor.enrolledFactors.length > 0) {
            addReadonlyGetter(this, 'multiFactor', multiFactor);
        }
    }
    toJSON() {
        const json = {
            uid: this.uid,
            email: this.email,
            emailVerified: this.emailVerified,
            displayName: this.displayName,
            photoURL: this.photoURL,
            phoneNumber: this.phoneNumber,
            disabled: this.disabled,
            metadata: this.metadata.toJSON(),
            passwordHash: this.passwordHash,
            passwordSalt: this.passwordSalt,
            customClaims: deepCopy(this.customClaims),
            tokensValidAfterTime: this.tokensValidAfterTime,
            tenantId: this.tenantId
        };
        if (this.multiFactor) {
            json.multiFactor = this.multiFactor.toJSON();
        }
        json.providerData = [];
        for (const entry of this.providerData) {
            json.providerData.push(entry.toJSON());
        }
        return json;
    }
}
