import { NextResponse } from 'next/server';
import { getFirebaseAuth } from '../../auth/index.js';
import { debug } from '../../debug/index.js';
import { getCookiesTokens, getRequestCookiesTokens } from '../tokens.js';
import { getReferer } from '../utils.js';
import { AuthCookies } from './AuthCookies.js';
import { RequestCookiesProvider } from './parser/RequestCookiesProvider.js';
import { CookieExpirationFactory } from './expiration/CookieExpirationFactory.js';
import { CookieRemoverFactory } from './remover/CookieRemoverFactory.js';
export async function appendAuthCookies(headers, response, tokens, options) {
    debug('Updating response headers with authenticated cookies');
    const authCookies = new AuthCookies(RequestCookiesProvider.fromHeaders(headers), options);
    await authCookies.setAuthHeaders(tokens, response.headers);
}
export async function setAuthCookies(headers, options) {
    const { getCustomIdAndRefreshTokens } = getFirebaseAuth({
        serviceAccount: options.serviceAccount,
        apiKey: options.apiKey,
        tenantId: options.tenantId
    });
    const authHeader = options.authorizationHeaderName ?? 'Authorization';
    const token = headers.get(authHeader)?.split(' ')[1] ?? '';
    if (!token) {
        const response = new NextResponse(JSON.stringify({ success: false, message: 'Missing token' }), {
            status: 400,
            headers: { 'content-type': 'application/json' }
        });
        return response;
    }
    const appCheckToken = headers.get('X-Firebase-AppCheck') ?? undefined;
    const referer = getReferer(headers) ?? '';
    const customTokens = await getCustomIdAndRefreshTokens(token, {
        appCheckToken,
        referer,
        dynamicCustomClaimsKeys: options.dynamicCustomClaimsKeys
    });
    debug('Successfully generated custom tokens');
    const response = new NextResponse(JSON.stringify({ success: true }), {
        status: 200,
        headers: { 'content-type': 'application/json' }
    });
    await appendAuthCookies(headers, response, customTokens, options);
    return response;
}
export function removeServerCookies(cookies, options) {
    const remover = CookieRemoverFactory.fromRequestCookies(cookies, RequestCookiesProvider.fromRequestCookies(cookies), options.cookieName);
    return remover.removeCookies();
}
export function removeCookies(headers, response, options) {
    const expiration = CookieExpirationFactory.fromHeaders(response.headers, RequestCookiesProvider.fromHeaders(headers), options.cookieName);
    return expiration.expireCookies(options.cookieSerializeOptions);
}
export function removeAuthCookies(headers, options) {
    const response = new NextResponse(JSON.stringify({ success: true }), {
        status: 200,
        headers: { 'content-type': 'application/json' }
    });
    removeCookies(headers, response, options);
    debug('Updating response with empty authentication cookie headers', {
        cookieName: options.cookieName
    });
    return response;
}
export async function verifyApiCookies(cookies, headers, options) {
    const tokens = await getCookiesTokens(cookies, options);
    const { verifyAndRefreshExpiredIdToken } = getFirebaseAuth({
        serviceAccount: options.serviceAccount,
        apiKey: options.apiKey,
        tenantId: options.tenantId
    });
    const verifyTokenResult = await verifyAndRefreshExpiredIdToken(tokens, {
        referer: headers.referer ?? ''
    });
    return verifyTokenResult;
}
export async function verifyNextCookies(cookies, headers, options) {
    const { verifyAndRefreshExpiredIdToken } = getFirebaseAuth({
        serviceAccount: options.serviceAccount,
        apiKey: options.apiKey,
        tenantId: options.tenantId,
        enableCustomToken: options.enableCustomToken
    });
    const referer = getReferer(headers) ?? '';
    const tokens = await getRequestCookiesTokens(cookies, options);
    const verifyTokenResult = await verifyAndRefreshExpiredIdToken(tokens, {
        referer
    });
    return verifyTokenResult;
}
export async function refreshNextCookies(cookies, headers, options) {
    const { handleTokenRefresh } = getFirebaseAuth({
        serviceAccount: options.serviceAccount,
        apiKey: options.apiKey,
        tenantId: options.tenantId
    });
    const referer = getReferer(headers) ?? '';
    const tokens = await getRequestCookiesTokens(cookies, options);
    const tokenRefreshResult = await handleTokenRefresh(tokens.refreshToken, {
        referer,
        enableCustomToken: options.enableCustomToken
    });
    return {
        idToken: tokenRefreshResult.idToken,
        refreshToken: tokenRefreshResult.refreshToken,
        customToken: tokenRefreshResult.customToken,
        decodedIdToken: tokenRefreshResult.decodedIdToken
    };
}
export async function refreshCredentials(request, options, responseFactory) {
    const tokens = await refreshNextCookies(request.cookies, request.headers, options);
    const cookies = new AuthCookies(RequestCookiesProvider.fromHeaders(request.headers), options);
    await cookies.setAuthCookies(tokens, request.cookies);
    const responseOrPromise = responseFactory({
        headers: request.headers,
        tokens
    });
    const response = responseOrPromise instanceof Promise
        ? await responseOrPromise
        : responseOrPromise;
    await cookies.setAuthHeaders(tokens, response.headers);
    return response;
}
export async function refreshNextResponseCookiesWithToken(idToken, request, response, options) {
    const appCheckToken = request.headers.get('X-Firebase-AppCheck') ?? undefined;
    const referer = getReferer(request.headers) ?? '';
    const { getCustomIdAndRefreshTokens } = getFirebaseAuth({
        serviceAccount: options.serviceAccount,
        apiKey: options.apiKey,
        tenantId: options.tenantId
    });
    const customTokens = await getCustomIdAndRefreshTokens(idToken, {
        appCheckToken,
        referer
    });
    await appendAuthCookies(request.headers, response, customTokens, options);
    return response;
}
export async function refreshCookiesWithIdToken(idToken, headers, cookies, options) {
    const appCheckToken = headers.get('X-Firebase-AppCheck') ?? undefined;
    const referer = getReferer(headers) ?? '';
    const { getCustomIdAndRefreshTokens } = getFirebaseAuth({
        serviceAccount: options.serviceAccount,
        apiKey: options.apiKey,
        tenantId: options.tenantId
    });
    const customTokens = await getCustomIdAndRefreshTokens(idToken, {
        appCheckToken,
        referer
    });
    const authCookies = new AuthCookies(RequestCookiesProvider.fromHeaders(headers), options);
    await authCookies.setAuthCookies(customTokens, cookies);
}
export async function refreshNextResponseCookies(request, response, options) {
    const customTokens = await refreshNextCookies(request.cookies, request.headers, options);
    await appendAuthCookies(request.headers, response, customTokens, options);
    return response;
}
export async function refreshServerCookies(cookies, headers, options) {
    const customTokens = await refreshNextCookies(cookies, headers, options);
    const authCookies = new AuthCookies(RequestCookiesProvider.fromHeaders(headers), options);
    await authCookies.setAuthCookies(customTokens, cookies);
    await authCookies.setAuthHeaders(customTokens, headers);
}
export * from './types.js';
