import { ServiceAccountCredential } from './credential.js';
import { AuthError, AuthErrorCode } from './error.js';
import { EmulatorSigner, IAMSigner, ServiceAccountSigner } from './jwt/crypto-signer';
import { isNonNullObject } from './validator.js';
import { useEmulator } from './firebase.js';
const ONE_HOUR_IN_SECONDS = 60 * 60;
export const BLACKLISTED_CLAIMS = [
    'acr',
    'amr',
    'at_hash',
    'aud',
    'auth_time',
    'azp',
    'cnf',
    'c_hash',
    'exp',
    'iat',
    'iss',
    'jti',
    'nbf',
    'nonce'
];
const FIREBASE_AUDIENCE = 'https://identitytoolkit.googleapis.com/google.identity.identitytoolkit.v1.IdentityToolkit';
export class FirebaseTokenGenerator {
    signer;
    constructor(signer) {
        this.signer = signer;
    }
    createCustomToken(uid, developerClaims) {
        let errorMessage;
        if (uid.length > 128) {
            errorMessage =
                '`uid` argument must a uid with less than or equal to 128 characters.';
        }
        else if (!FirebaseTokenGenerator.isDeveloperClaimsValid_(developerClaims)) {
            errorMessage =
                '`developerClaims` argument must be a valid, non-null object containing the developer claims.';
        }
        if (errorMessage) {
            throw new AuthError(AuthErrorCode.INVALID_ARGUMENT, errorMessage);
        }
        const claims = {};
        if (typeof developerClaims !== 'undefined') {
            for (const key in developerClaims) {
                if (Object.prototype.hasOwnProperty.call(developerClaims, key)) {
                    if (BLACKLISTED_CLAIMS.indexOf(key) !== -1) {
                        throw new AuthError(AuthErrorCode.INVALID_ARGUMENT, `Developer claim "${key}" is reserved and cannot be specified.`);
                    }
                    claims[key] = developerClaims[key];
                }
            }
        }
        return this.signer.getAccountId().then(async (account) => {
            const iat = Math.floor(Date.now() / 1000);
            const body = {
                aud: FIREBASE_AUDIENCE,
                iat,
                exp: iat + ONE_HOUR_IN_SECONDS,
                iss: account,
                sub: account,
                uid
            };
            if (Object.keys(claims).length > 0) {
                body.claims = claims;
            }
            return this.signer.sign(body);
        });
    }
    static isDeveloperClaimsValid_(developerClaims) {
        if (typeof developerClaims === 'undefined') {
            return true;
        }
        return isNonNullObject(developerClaims);
    }
}
export function cryptoSignerFromCredential(credential, tenantId, serviceAccountId) {
    if (credential instanceof ServiceAccountCredential) {
        return new ServiceAccountSigner(credential, tenantId);
    }
    return new IAMSigner(credential, tenantId, serviceAccountId);
}
export function createFirebaseTokenGenerator(credential, tenantId, serviceAccountId) {
    if (useEmulator()) {
        return new FirebaseTokenGenerator(new EmulatorSigner(tenantId));
    }
    const signer = cryptoSignerFromCredential(credential, tenantId, serviceAccountId);
    return new FirebaseTokenGenerator(signer);
}
