"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AppCheckTokenVerifier = void 0;
const jose_1 = require("jose");
const verify_js_1 = require("../auth/jwt/verify.js");
const signature_verifier_1 = require("../auth/signature-verifier");
const api_client_js_1 = require("./api-client.js");
const APP_CHECK_ISSUER = 'https://firebaseappcheck.googleapis.com/';
const JWKS_URL = 'https://firebaseappcheck.googleapis.com/v1/jwks';
class AppCheckTokenVerifier {
    credential;
    signatureVerifier;
    constructor(credential) {
        this.credential = credential;
        this.signatureVerifier = new signature_verifier_1.JWKSSignatureVerifier(JWKS_URL);
    }
    async verifyToken(token, options) {
        const projectId = await this.credential.getProjectId();
        const decoded = await this.decodeAndVerify(token, projectId, options);
        const decodedAppCheckToken = decoded.payload;
        decodedAppCheckToken.app_id = decodedAppCheckToken.sub;
        return decodedAppCheckToken;
    }
    async decodeAndVerify(token, projectId, options) {
        const header = (0, jose_1.decodeProtectedHeader)(token);
        const payload = (0, jose_1.decodeJwt)(token);
        this.verifyContent({ header, payload }, projectId);
        await this.verifySignature(token, options);
        return { header, payload };
    }
    verifyContent(fullDecodedToken, projectId) {
        const header = fullDecodedToken.header;
        const payload = fullDecodedToken.payload;
        const projectIdMatchMessage = ' Make sure the App Check token comes from the same ' +
            'Firebase project as the service account used to authenticate this SDK.';
        const scopedProjectId = `projects/${projectId}`;
        let errorMessage;
        if (header.alg !== verify_js_1.ALGORITHM_RS256) {
            errorMessage =
                'The provided App Check token has incorrect algorithm. Expected "' +
                    verify_js_1.ALGORITHM_RS256 +
                    '" but got ' +
                    '"' +
                    header.alg +
                    '".';
        }
        else if (!payload.aud?.includes(scopedProjectId)) {
            errorMessage =
                'The provided App Check token has incorrect "aud" (audience) claim. Expected "' +
                    scopedProjectId +
                    '" but got "' +
                    payload.aud +
                    '".' +
                    projectIdMatchMessage;
        }
        else if (typeof payload.iss !== 'string' ||
            !payload.iss.startsWith(APP_CHECK_ISSUER)) {
            errorMessage =
                'The provided App Check token has incorrect "iss" (issuer) claim.';
        }
        else if (typeof payload.sub !== 'string') {
            errorMessage =
                'The provided App Check token has no "sub" (subject) claim.';
        }
        else if (payload.sub === '') {
            errorMessage =
                'The provided App Check token has an empty string "sub" (subject) claim.';
        }
        if (errorMessage) {
            throw new api_client_js_1.FirebaseAppCheckError('invalid-argument', errorMessage);
        }
    }
    verifySignature(jwtToken, options) {
        return this.signatureVerifier
            .verify(jwtToken, options)
            .catch((error) => {
            throw this.mapJwtErrorToAppCheckError(error);
        });
    }
    mapJwtErrorToAppCheckError(error) {
        if (error instanceof jose_1.errors.JWTExpired) {
            const errorMessage = 'The provided App Check token has expired. Get a fresh App Check token' +
                ' from your client app and try again.';
            return new api_client_js_1.FirebaseAppCheckError('app-check-token-expired', errorMessage);
        }
        else if (error instanceof jose_1.errors.JWSSignatureVerificationFailed) {
            const errorMessage = 'The provided App Check token has invalid signature.';
            return new api_client_js_1.FirebaseAppCheckError('invalid-argument', errorMessage);
        }
        else if (error instanceof jose_1.errors.JWKSNoMatchingKey) {
            const errorMessage = 'The provided App Check token has "kid" claim which does not ' +
                'correspond to a known public key. Most likely the provided App Check token ' +
                'is expired, so get a fresh token from your client app and try again.';
            return new api_client_js_1.FirebaseAppCheckError('invalid-argument', errorMessage);
        }
        return new api_client_js_1.FirebaseAppCheckError('invalid-argument', error.message);
    }
}
exports.AppCheckTokenVerifier = AppCheckTokenVerifier;
