"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.IAMSigner = exports.ServiceAccountSigner = exports.EmulatorSigner = void 0;
exports.createEmulatorToken = createEmulatorToken;
const jose_1 = require("jose");
const utils_js_1 = require("../utils.js");
const sign_js_1 = require("./sign.js");
const verify_js_1 = require("./verify.js");
function createEmulatorToken(payload) {
    const header = {
        alg: 'none',
        typ: 'JWT'
    };
    return `${jose_1.base64url.encode(JSON.stringify(header))}.${jose_1.base64url.encode(JSON.stringify(payload))}.${jose_1.base64url.encode('')}`;
}
class EmulatorSigner {
    tenantId;
    constructor(tenantId) {
        this.tenantId = tenantId;
    }
    async sign(payload) {
        if (this.tenantId) {
            payload.tenant_id = this.tenantId;
        }
        return createEmulatorToken(payload);
    }
    getAccountId() {
        return Promise.resolve('firebase-auth-emulator@example.com');
    }
}
exports.EmulatorSigner = EmulatorSigner;
class ServiceAccountSigner {
    credential;
    tenantId;
    constructor(credential, tenantId) {
        this.credential = credential;
        this.tenantId = tenantId;
    }
    async sign(payload) {
        if (this.tenantId) {
            payload.tenant_id = this.tenantId;
        }
        return (0, sign_js_1.sign)({ payload, privateKey: this.credential.privateKey });
    }
    getAccountId() {
        return Promise.resolve(this.credential.clientEmail);
    }
}
exports.ServiceAccountSigner = ServiceAccountSigner;
class IAMSigner {
    algorithm = verify_js_1.ALGORITHM_RS256;
    credential;
    tenantId;
    serviceAccountId;
    constructor(credential, tenantId, serviceAccountId) {
        this.credential = credential;
        this.tenantId = tenantId;
        this.serviceAccountId = serviceAccountId;
    }
    async sign(payload) {
        if (this.tenantId) {
            payload.tenant_id = this.tenantId;
        }
        const serviceAccount = await this.getAccountId();
        const accessToken = await this.credential.getAccessToken();
        return (0, sign_js_1.signBlob)({
            accessToken: accessToken.accessToken,
            serviceAccountId: serviceAccount,
            payload
        });
    }
    async getAccountId() {
        if (this.serviceAccountId) {
            return this.serviceAccountId;
        }
        const token = await this.credential.getAccessToken();
        const url = 'http://metadata/computeMetadata/v1/instance/service-accounts/default/email';
        const request = {
            method: 'GET',
            headers: {
                'Metadata-Flavor': 'Google',
                Authorization: `Bearer ${token.accessToken}`
            }
        };
        return (this.serviceAccountId = await (0, utils_js_1.fetchText)(url, request));
    }
}
exports.IAMSigner = IAMSigner;
