"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PublicKeySignatureVerifier = exports.JWKSSignatureVerifier = exports.UrlKeyFetcher = void 0;
exports.fetchPublicKey = fetchPublicKey;
const jose_1 = require("jose");
const index_js_1 = require("../debug/index.js");
const error_js_1 = require("./error.js");
const firebase_js_1 = require("./firebase.js");
const verify_js_1 = require("./jwt/verify.js");
const validator_js_1 = require("./validator.js");
function getExpiresAt(res) {
    if (!res.headers.has('cache-control')) {
        return 0;
    }
    const cacheControlHeader = res.headers.get('cache-control');
    const parts = cacheControlHeader.split(',');
    const maxAge = parts.reduce((acc, part) => {
        const subParts = part.trim().split('=');
        if (subParts[0] === 'max-age') {
            return +subParts[1];
        }
        return acc;
    }, 0);
    return Date.now() + maxAge * 1000;
}
const keyResponseCache = new Map();
class UrlKeyFetcher {
    clientCertUrl;
    constructor(clientCertUrl) {
        this.clientCertUrl = clientCertUrl;
        if (!(0, validator_js_1.isURL)(clientCertUrl)) {
            throw new Error('The provided public client certificate URL is not a valid URL.');
        }
    }
    async fetchPublicKeysResponse(url) {
        const res = await fetch(url);
        const headers = {};
        res.headers.forEach((value, key) => {
            headers[key] = value;
        });
        (0, index_js_1.debug)('Public keys fetched', {
            responseHeaders: headers,
            cryptoRuntime: jose_1.cryptoRuntime
        });
        if (!res.ok) {
            let errorMessage = 'Error fetching public keys for Google certs: ';
            const data = await res.json();
            if (data.error) {
                errorMessage += `${data.error}`;
                if (data.error_description) {
                    errorMessage += ' (' + data.error_description + ')';
                }
            }
            else {
                errorMessage += `${await res.text()}`;
            }
            throw new Error(errorMessage);
        }
        const data = await res.json();
        if (data.error) {
            throw new Error('Error fetching public keys for Google certs: ' + data.error);
        }
        const expiresAt = getExpiresAt(res);
        return {
            keys: data,
            expiresAt
        };
    }
    async fetchAndCachePublicKeys(url) {
        (0, index_js_1.debug)('No public keys found in cache. Fetching public keys from Google...', {
            cryptoRuntime: jose_1.cryptoRuntime
        });
        const response = await this.fetchPublicKeysResponse(url);
        keyResponseCache.set(url.toString(), response);
        (0, index_js_1.debug)('Public keys cached', {
            cacheKey: url.toString(),
            expiresAt: response.expiresAt,
            cryptoRuntime: jose_1.cryptoRuntime
        });
        return response.keys;
    }
    async fetchPublicKeys() {
        const url = new URL(this.clientCertUrl);
        const cachedResponse = keyResponseCache.get(url.toString());
        if (!cachedResponse) {
            return this.fetchAndCachePublicKeys(url);
        }
        const { keys, expiresAt } = cachedResponse;
        const now = Date.now();
        (0, index_js_1.debug)('Get public keys from cache', {
            expiresAt,
            now,
            cryptoRuntime: jose_1.cryptoRuntime
        });
        if (expiresAt <= now) {
            return this.fetchAndCachePublicKeys(url);
        }
        return keys;
    }
}
exports.UrlKeyFetcher = UrlKeyFetcher;
class JWKSSignatureVerifier {
    options;
    jwksUrl;
    constructor(jwksUrl, options) {
        this.options = options;
        if (!(0, validator_js_1.isURL)(jwksUrl)) {
            throw new Error('The provided JWKS URL is not a valid URL.');
        }
        this.jwksUrl = new URL(jwksUrl);
    }
    async getPublicKey(header) {
        const getKey = (0, jose_1.createRemoteJWKSet)(this.jwksUrl, this.options);
        return getKey(header);
    }
    async verify(token, options) {
        const header = (0, jose_1.decodeProtectedHeader)(token);
        try {
            await (0, verify_js_1.verify)(token, () => this.getPublicKey(header), options);
        }
        catch (e) {
            if (e instanceof jose_1.errors.JWKSMultipleMatchingKeys) {
                for await (const publicKey of e) {
                    try {
                        await (0, verify_js_1.verify)(token, () => Promise.resolve(publicKey), options);
                        return;
                    }
                    catch (innerError) {
                        if (innerError instanceof jose_1.errors.JWSSignatureVerificationFailed) {
                            continue;
                        }
                        throw innerError;
                    }
                }
                throw new jose_1.errors.JWSSignatureVerificationFailed();
            }
            throw e;
        }
    }
}
exports.JWKSSignatureVerifier = JWKSSignatureVerifier;
class PublicKeySignatureVerifier {
    keyFetcher;
    constructor(keyFetcher) {
        this.keyFetcher = keyFetcher;
        if (!(0, validator_js_1.isNonNullObject)(keyFetcher)) {
            throw new Error('The provided key fetcher is not an object or null.');
        }
    }
    static withCertificateUrl(clientCertUrl) {
        return new PublicKeySignatureVerifier(new UrlKeyFetcher(clientCertUrl));
    }
    async getPublicKey(header) {
        if ((0, firebase_js_1.useEmulator)()) {
            return { type: 'none' };
        }
        return fetchPublicKey(this.keyFetcher, header).then(verify_js_1.getPublicCryptoKey);
    }
    async verify(token, options) {
        const header = (0, jose_1.decodeProtectedHeader)(token);
        try {
            await (0, verify_js_1.verify)(token, () => this.getPublicKey(header), options);
        }
        catch (e) {
            if (e instanceof error_js_1.AuthError && e.code === error_js_1.AuthErrorCode.NO_KID_IN_HEADER) {
                await this.verifyWithoutKid(token, options);
                return;
            }
            throw e;
        }
    }
    async verifyWithoutKid(token, options) {
        const publicKeys = await this.keyFetcher.fetchPublicKeys();
        return this.verifyWithAllKeys(token, publicKeys, options);
    }
    async verifyWithAllKeys(token, keys, options) {
        const promises = [];
        Object.values(keys).forEach((key) => {
            const promise = (0, verify_js_1.verify)(token, async () => (0, verify_js_1.getPublicCryptoKey)(key), options)
                .then(() => true)
                .catch((error) => {
                if (error instanceof jose_1.errors.JWTExpired) {
                    throw error;
                }
                return false;
            });
            promises.push(promise);
        });
        return Promise.all(promises).then((result) => {
            if (result.every((r) => r === false)) {
                throw new error_js_1.AuthError(error_js_1.AuthErrorCode.INVALID_SIGNATURE);
            }
        });
    }
}
exports.PublicKeySignatureVerifier = PublicKeySignatureVerifier;
async function fetchPublicKey(fetcher, header) {
    if (!header.kid) {
        throw new error_js_1.AuthError(error_js_1.AuthErrorCode.NO_KID_IN_HEADER);
    }
    const kid = header.kid;
    const publicKeys = await fetcher.fetchPublicKeys();
    if (!Object.prototype.hasOwnProperty.call(publicKeys, kid)) {
        throw new error_js_1.AuthError(error_js_1.AuthErrorCode.NO_MATCHING_KID);
    }
    return publicKeys[kid];
}
