"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FirebaseTokenGenerator = exports.BLACKLISTED_CLAIMS = void 0;
exports.cryptoSignerFromCredential = cryptoSignerFromCredential;
exports.createFirebaseTokenGenerator = createFirebaseTokenGenerator;
const credential_js_1 = require("./credential.js");
const error_js_1 = require("./error.js");
const crypto_signer_1 = require("./jwt/crypto-signer");
const validator_js_1 = require("./validator.js");
const firebase_js_1 = require("./firebase.js");
const ONE_HOUR_IN_SECONDS = 60 * 60;
exports.BLACKLISTED_CLAIMS = [
    'acr',
    'amr',
    'at_hash',
    'aud',
    'auth_time',
    'azp',
    'cnf',
    'c_hash',
    'exp',
    'iat',
    'iss',
    'jti',
    'nbf',
    'nonce'
];
const FIREBASE_AUDIENCE = 'https://identitytoolkit.googleapis.com/google.identity.identitytoolkit.v1.IdentityToolkit';
class FirebaseTokenGenerator {
    signer;
    constructor(signer) {
        this.signer = signer;
    }
    createCustomToken(uid, developerClaims) {
        let errorMessage;
        if (uid.length > 128) {
            errorMessage =
                '`uid` argument must a uid with less than or equal to 128 characters.';
        }
        else if (!FirebaseTokenGenerator.isDeveloperClaimsValid_(developerClaims)) {
            errorMessage =
                '`developerClaims` argument must be a valid, non-null object containing the developer claims.';
        }
        if (errorMessage) {
            throw new error_js_1.AuthError(error_js_1.AuthErrorCode.INVALID_ARGUMENT, errorMessage);
        }
        const claims = {};
        if (typeof developerClaims !== 'undefined') {
            for (const key in developerClaims) {
                if (Object.prototype.hasOwnProperty.call(developerClaims, key)) {
                    if (exports.BLACKLISTED_CLAIMS.indexOf(key) !== -1) {
                        throw new error_js_1.AuthError(error_js_1.AuthErrorCode.INVALID_ARGUMENT, `Developer claim "${key}" is reserved and cannot be specified.`);
                    }
                    claims[key] = developerClaims[key];
                }
            }
        }
        return this.signer.getAccountId().then(async (account) => {
            const iat = Math.floor(Date.now() / 1000);
            const body = {
                aud: FIREBASE_AUDIENCE,
                iat,
                exp: iat + ONE_HOUR_IN_SECONDS,
                iss: account,
                sub: account,
                uid
            };
            if (Object.keys(claims).length > 0) {
                body.claims = claims;
            }
            return this.signer.sign(body);
        });
    }
    static isDeveloperClaimsValid_(developerClaims) {
        if (typeof developerClaims === 'undefined') {
            return true;
        }
        return (0, validator_js_1.isNonNullObject)(developerClaims);
    }
}
exports.FirebaseTokenGenerator = FirebaseTokenGenerator;
function cryptoSignerFromCredential(credential, tenantId, serviceAccountId) {
    if (credential instanceof credential_js_1.ServiceAccountCredential) {
        return new crypto_signer_1.ServiceAccountSigner(credential, tenantId);
    }
    return new crypto_signer_1.IAMSigner(credential, tenantId, serviceAccountId);
}
function createFirebaseTokenGenerator(credential, tenantId, serviceAccountId) {
    if ((0, firebase_js_1.useEmulator)()) {
        return new FirebaseTokenGenerator(new crypto_signer_1.EmulatorSigner(tenantId));
    }
    const signer = cryptoSignerFromCredential(credential, tenantId, serviceAccountId);
    return new FirebaseTokenGenerator(signer);
}
