"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FirebaseTokenVerifier = void 0;
exports.createIdTokenVerifier = createIdTokenVerifier;
const jose_1 = require("jose");
const error_js_1 = require("./error.js");
const firebase_js_1 = require("./firebase.js");
const verify_js_1 = require("./jwt/verify.js");
const signature_verifier_1 = require("./signature-verifier");
const utils_js_1 = require("./utils.js");
const validator_js_1 = require("./validator.js");
class FirebaseTokenVerifier {
    issuer;
    projectId;
    tenantId;
    signatureVerifier;
    constructor(clientCertUrl, issuer, projectId, tenantId) {
        this.issuer = issuer;
        this.projectId = projectId;
        this.tenantId = tenantId;
        if (!(0, validator_js_1.isURL)(clientCertUrl)) {
            throw new error_js_1.AuthError(error_js_1.AuthErrorCode.INVALID_ARGUMENT, 'The provided public client certificate URL is an invalid URL.');
        }
        this.signatureVerifier =
            signature_verifier_1.PublicKeySignatureVerifier.withCertificateUrl(clientCertUrl);
    }
    async verifyJWT(jwtToken, options) {
        const decoded = await this.decodeAndVerify(jwtToken, this.projectId, options);
        const decodedIdToken = (0, utils_js_1.mapJwtPayloadToDecodedIdToken)(decoded.payload);
        if (this.tenantId && decodedIdToken.firebase.tenant !== this.tenantId) {
            throw new error_js_1.AuthError(error_js_1.AuthErrorCode.MISMATCHING_TENANT_ID);
        }
        return decodedIdToken;
    }
    async decodeAndVerify(token, projectId, options) {
        const header = (0, jose_1.decodeProtectedHeader)(token);
        const payload = (0, jose_1.decodeJwt)(token);
        this.verifyContent({ header, payload }, projectId);
        await this.verifySignature(token, options);
        return { header, payload };
    }
    verifyContent(fullDecodedToken, projectId) {
        const header = fullDecodedToken && fullDecodedToken.header;
        const payload = fullDecodedToken && fullDecodedToken.payload;
        let errorMessage;
        if (!(0, firebase_js_1.useEmulator)() && typeof header.kid === 'undefined') {
            const isCustomToken = payload.aud === firebase_js_1.FIREBASE_AUDIENCE;
            if (isCustomToken) {
                errorMessage = `idToken was expected, but custom token was provided`;
            }
            else {
                errorMessage = `idToken has no "kid" claim.`;
            }
        }
        else if (!(0, firebase_js_1.useEmulator)() && header.alg !== verify_js_1.ALGORITHM_RS256) {
            errorMessage = `Incorrect algorithm. ${verify_js_1.ALGORITHM_RS256} expected, ${header.alg} provided`;
        }
        else if (payload.iss !== this.issuer + projectId) {
            errorMessage = `idToken has incorrect "iss" (issuer) claim. Expected ${this.issuer}${projectId}, but got ${payload.iss}`;
        }
        else if (payload.sub === '') {
            errorMessage = `idToken has an empty string "sub" (subject) claim.`;
        }
        else if (typeof payload.sub === 'string' && payload.sub.length > 128) {
            errorMessage = `idToken has "sub" (subject) claim longer than 128 characters.`;
        }
        if (errorMessage) {
            throw new error_js_1.AuthError(error_js_1.AuthErrorCode.INVALID_ARGUMENT, errorMessage);
        }
    }
    verifySignature(jwtToken, options) {
        return this.signatureVerifier.verify(jwtToken, options).catch((error) => {
            throw this.mapJoseErrorToAuthError(error);
        });
    }
    mapJoseErrorToAuthError(error) {
        if (error instanceof jose_1.errors.JWTExpired) {
            return error_js_1.AuthError.fromError(error, error_js_1.AuthErrorCode.TOKEN_EXPIRED, error.message);
        }
        if (error instanceof jose_1.errors.JWSSignatureVerificationFailed) {
            return error_js_1.AuthError.fromError(error, error_js_1.AuthErrorCode.INVALID_SIGNATURE);
        }
        if (error instanceof error_js_1.AuthError) {
            return error;
        }
        return error_js_1.AuthError.fromError(error, error_js_1.AuthErrorCode.INTERNAL_ERROR, error.message);
    }
}
exports.FirebaseTokenVerifier = FirebaseTokenVerifier;
function createIdTokenVerifier(projectId, tenantId) {
    return new FirebaseTokenVerifier(firebase_js_1.CLIENT_CERT_URL, 'https://securetoken.google.com/', projectId, tenantId);
}
