"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AuthCookies = void 0;
const CookieBuilderFactory_js_1 = require("./builder/CookieBuilderFactory.js");
const CookieParserFactory_js_1 = require("./parser/CookieParserFactory.js");
const MultipleCookieExpiration_js_1 = require("./expiration/MultipleCookieExpiration.js");
const SingleCookieExpiration_js_1 = require("./expiration/SingleCookieExpiration.js");
const CookieSetterFactory_js_1 = require("./setter/CookieSetterFactory.js");
const NextApiResponseHeadersCookieSetter_js_1 = require("./setter/NextApiResponseHeadersCookieSetter.js");
class AuthCookies {
    provider;
    options;
    builder;
    targetCookies = null;
    constructor(provider, options) {
        this.provider = provider;
        this.options = options;
        this.builder = CookieBuilderFactory_js_1.CookieBuilderFactory.fromOptions(options);
    }
    shouldClearMultipleCookies() {
        return (!this.options.enableMultipleCookies &&
            (CookieParserFactory_js_1.CookieParserFactory.hasMultipleCookies(this.provider, this.options.cookieName) ||
                CookieParserFactory_js_1.CookieParserFactory.hasLegacyMultipleCookies(this.provider, this.options.cookieName)));
    }
    shouldClearCustomTokenCookie() {
        return (!this.options.enableCustomToken &&
            CookieParserFactory_js_1.CookieParserFactory.hasCustomTokenCookie(this.provider, this.options.cookieName));
    }
    shouldClearSingleCookie() {
        const hasSingleCookie = Boolean(this.provider.get(this.options.cookieName));
        return this.options.enableMultipleCookies && hasSingleCookie;
    }
    clearUnusedCookies(setter) {
        if (this.shouldClearMultipleCookies()) {
            const expiration = new MultipleCookieExpiration_js_1.MultipleCookieExpiration(this.options.cookieName, setter);
            expiration.expireCookies(this.options.cookieSerializeOptions);
        }
        else if (this.shouldClearCustomTokenCookie()) {
            const expiration = new MultipleCookieExpiration_js_1.MultipleCookieExpiration(this.options.cookieName, setter);
            expiration.expireCustomCookie(this.options.cookieSerializeOptions);
        }
        if (this.shouldClearSingleCookie()) {
            const expiration = new SingleCookieExpiration_js_1.SingleCookieExpiration(this.options.cookieName, setter);
            expiration.expireCookies(this.options.cookieSerializeOptions);
        }
    }
    async getCookies(tokens) {
        const tokensToSave = this.options.enableCustomToken
            ? tokens
            : { idToken: tokens.idToken, refreshToken: tokens.refreshToken };
        if (this.targetCookies) {
            return this.targetCookies;
        }
        return (this.targetCookies = await this.builder.buildCookies(tokensToSave));
    }
    async setAuthCookies(tokens, requestCookies) {
        const cookies = await this.getCookies(tokens);
        const setter = CookieSetterFactory_js_1.CookieSetterFactory.fromRequestCookies(requestCookies);
        this.clearUnusedCookies(setter);
        setter.setCookies(cookies, this.options.cookieSerializeOptions);
    }
    async setAuthHeaders(tokens, headers) {
        const cookies = await this.getCookies(tokens);
        const setter = CookieSetterFactory_js_1.CookieSetterFactory.fromHeaders(headers);
        this.clearUnusedCookies(setter);
        setter.setCookies(cookies, this.options.cookieSerializeOptions);
    }
    async setAuthNextApiResponseHeaders(tokens, response) {
        const cookies = await this.getCookies(tokens);
        const setter = new NextApiResponseHeadersCookieSetter_js_1.NextApiResponseCookieSetter(response);
        this.clearUnusedCookies(setter);
        setter.setCookies(cookies, this.options.cookieSerializeOptions);
    }
}
exports.AuthCookies = AuthCookies;
