"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CookieParserFactory = void 0;
const error_js_1 = require("../../../auth/error.js");
const index_js_1 = require("../../../debug/index.js");
const MultipleCookiesParser_js_1 = require("./MultipleCookiesParser.js");
const ObjectCookiesProvider_js_1 = require("./ObjectCookiesProvider.js");
const RequestCookiesProvider_js_1 = require("./RequestCookiesProvider.js");
const SingleCookieParser_js_1 = require("./SingleCookieParser.js");
class CookieParserFactory {
    static hasMultipleCookies(provider, cookieName) {
        return ['id', 'refresh', 'sig']
            .map((it) => `${cookieName}.${it}`)
            .every((it) => Boolean(provider.get(it)));
    }
    static hasCustomTokenCookie(provider, cookieName) {
        return Boolean(provider.get(`${cookieName}.custom`));
    }
    static hasLegacyMultipleCookies(provider, cookieName) {
        return (['custom', 'sig']
            .map((it) => `${cookieName}.${it}`)
            .every((it) => Boolean(provider.get(it))) &&
            provider.get(cookieName)?.includes(':'));
    }
    static getCompatibleProvider(legacyProvider, options) {
        const legacyToken = legacyProvider.get(options.cookieName);
        const [idToken, refreshToken] = legacyToken?.split(':') ?? [];
        const adaptedCookies = {
            [`${options.cookieName}.id`]: idToken,
            [`${options.cookieName}.refresh`]: refreshToken,
            [`${options.cookieName}.custom`]: legacyProvider.get(`${options.cookieName}.custom`),
            [`${options.cookieName}.sig`]: legacyProvider.get(`${options.cookieName}.sig`)
        };
        return new ObjectCookiesProvider_js_1.ObjectCookiesProvider(adaptedCookies);
    }
    static fromProvider(provider, options) {
        const singleCookie = provider.get(options.cookieName);
        const hasLegacyCookie = singleCookie?.includes(':');
        const enableMultipleCookies = CookieParserFactory.hasMultipleCookies(provider, options.cookieName);
        if (enableMultipleCookies) {
            return new MultipleCookiesParser_js_1.MultipleCookiesParser(provider, options.cookieName, options.cookieSignatureKeys);
        }
        if (CookieParserFactory.hasLegacyMultipleCookies(provider, options.cookieName)) {
            return new MultipleCookiesParser_js_1.MultipleCookiesParser(CookieParserFactory.getCompatibleProvider(provider, options), options.cookieName, options.cookieSignatureKeys);
        }
        if (hasLegacyCookie) {
            (0, index_js_1.debug)("Authentication cookie is in multiple cookie format, but lacks signature and custom cookies. Clear your browser cookies and try again. If the issue keeps happening and you're using `enableMultipleCookies` option, make sure that server returns all required cookies: https://next-firebase-auth-edge-docs.vercel.app/docs/usage/middleware#multiple-cookies");
            throw new error_js_1.InvalidTokenError(error_js_1.InvalidTokenReason.INVALID_CREDENTIALS);
        }
        return new SingleCookieParser_js_1.SingleCookieParser(provider, options.cookieName, options.cookieSignatureKeys);
    }
    static fromRequestCookies(cookies, options) {
        const provider = RequestCookiesProvider_js_1.RequestCookiesProvider.fromRequestCookies(cookies);
        return CookieParserFactory.fromProvider(provider, options);
    }
    static fromHeaders(headers, options) {
        const provider = RequestCookiesProvider_js_1.RequestCookiesProvider.fromHeaders(headers);
        return CookieParserFactory.fromProvider(provider, options);
    }
    static fromObject(cookies, options) {
        const provider = new ObjectCookiesProvider_js_1.ObjectCookiesProvider(cookies);
        return CookieParserFactory.fromProvider(provider, options);
    }
}
exports.CookieParserFactory = CookieParserFactory;
