"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.redirectToPath = redirectToPath;
exports.redirectToHome = redirectToHome;
exports.redirectToLogin = redirectToLogin;
exports.createAuthMiddlewareResponse = createAuthMiddlewareResponse;
exports.authMiddleware = authMiddleware;
const server_1 = require("next/server");
const error_js_1 = require("../auth/error.js");
const index_js_1 = require("../auth/index.js");
const index_js_2 = require("../debug/index.js");
const AuthCookies_js_1 = require("./cookies/AuthCookies.js");
const index_js_3 = require("./cookies/index.js");
const RequestCookiesProvider_js_1 = require("./cookies/parser/RequestCookiesProvider.js");
const verification_js_1 = require("./cookies/verification.js");
const refresh_token_js_1 = require("./refresh-token.js");
const tokens_js_1 = require("./tokens.js");
const utils_js_1 = require("./utils.js");
function redirectToPath(request, path, options = { shouldClearSearchParams: false }) {
    const url = request.nextUrl.clone();
    url.pathname = path;
    if (options.shouldClearSearchParams) {
        url.search = '';
    }
    return server_1.NextResponse.redirect(url);
}
function redirectToHome(request, options = {
    path: '/'
}) {
    return redirectToPath(request, options.path, { shouldClearSearchParams: true });
}
function doesRequestPathnameMatchPublicPath(request, publicPath) {
    if (typeof publicPath === 'string') {
        return publicPath === request.nextUrl.pathname;
    }
    return publicPath.test(request.nextUrl.pathname);
}
function doesRequestPathnameMatchOneOfPublicPaths(request, publicPaths) {
    return publicPaths.some((path) => doesRequestPathnameMatchPublicPath(request, path));
}
function redirectToLogin(request, options = {
    path: '/login',
    publicPaths: ['/login']
}) {
    const redirectKey = options.redirectParamKeyName || 'redirect';
    if (doesRequestPathnameMatchOneOfPublicPaths(request, options.publicPaths)) {
        return server_1.NextResponse.next();
    }
    const url = request.nextUrl.clone();
    url.pathname = options.path;
    url.search = `${redirectKey}=${request.nextUrl.pathname}${url.search}`;
    return server_1.NextResponse.redirect(url);
}
async function createAuthMiddlewareResponse(request, options) {
    if (request.nextUrl.pathname === options.loginPath) {
        return (0, index_js_3.setAuthCookies)(request.headers, {
            cookieName: options.cookieName,
            cookieSerializeOptions: options.cookieSerializeOptions,
            cookieSignatureKeys: options.cookieSignatureKeys,
            serviceAccount: options.serviceAccount,
            apiKey: options.apiKey,
            tenantId: options.tenantId,
            enableMultipleCookies: options.enableMultipleCookies,
            authorizationHeaderName: options.authorizationHeaderName,
            enableCustomToken: options.enableCustomToken,
            dynamicCustomClaimsKeys: options.dynamicCustomClaimsKeys
        });
    }
    if (request.nextUrl.pathname === options.logoutPath) {
        return (0, index_js_3.removeAuthCookies)(request.headers, {
            cookieName: options.cookieName,
            cookieSerializeOptions: options.cookieSerializeOptions
        });
    }
    if (options.refreshTokenPath &&
        request.nextUrl.pathname === options.refreshTokenPath) {
        return (0, refresh_token_js_1.refreshToken)(request, options);
    }
    return server_1.NextResponse.next();
}
const defaultInvalidTokenHandler = async () => server_1.NextResponse.next();
const defaultValidTokenHandler = async (_tokens, headers) => server_1.NextResponse.next({
    request: {
        headers
    }
});
function validateResponse(response) {
    if (!(0, verification_js_1.wasResponseDecoratedWithModifiedRequestHeaders)(response)) {
        console.warn(`– \x1b[33mwarn\x1b[0m next-firebase-auth-edge: NextResponse returned by handleValidToken was not decorated by modified request headers. This can cause token verification to happen multiple times in a single request. See: https://next-firebase-auth-edge-docs.vercel.app/docs/usage/middleware#middleware-token-verification-caching`);
    }
}
async function authMiddleware(request, options) {
    if (options.debug) {
        (0, index_js_2.enableDebugMode)();
    }
    (0, tokens_js_1.validateOptions)(options);
    const referer = (0, utils_js_1.getReferer)(request.headers) ?? '';
    const handleValidToken = options.handleValidToken ?? defaultValidTokenHandler;
    const handleError = options.handleError ?? defaultInvalidTokenHandler;
    const handleInvalidToken = options.handleInvalidToken ?? defaultInvalidTokenHandler;
    (0, verification_js_1.removeInternalVerifiedCookieIfExists)(request.cookies);
    (0, index_js_2.debug)('Handle request', { path: request.nextUrl.pathname });
    if ([options.loginPath, options.logoutPath, options.refreshTokenPath]
        .filter(Boolean)
        .includes(request.nextUrl.pathname)) {
        (0, index_js_2.debug)('Handle authentication API route');
        return createAuthMiddlewareResponse(request, options);
    }
    const { verifyIdToken, handleTokenRefresh, createAnonymousUser } = (0, index_js_1.getFirebaseAuth)({
        serviceAccount: options.serviceAccount,
        apiKey: options.apiKey,
        tenantId: options.tenantId
    });
    try {
        (0, index_js_2.debug)('Attempt to fetch request cookies tokens');
        const tokens = await (0, tokens_js_1.getRequestCookiesTokens)(request.cookies, options);
        return await (0, index_js_1.handleExpiredToken)(async () => {
            (0, index_js_2.debug)('Verifying user credentials...');
            const decodedToken = await verifyIdToken(tokens.idToken, {
                checkRevoked: options.checkRevoked,
                referer
            });
            (0, index_js_2.debug)('Credentials verified successfully');
            (0, verification_js_1.markCookiesAsVerified)(request.cookies);
            const response = await handleValidToken({
                token: tokens.idToken,
                decodedToken,
                customToken: tokens.customToken
            }, request.headers);
            (0, index_js_2.debug)('Successfully handled authenticated response');
            if (!response.headers.has('location')) {
                validateResponse(response);
            }
            return response;
        }, async () => {
            (0, index_js_2.debug)('Token has expired. Refreshing token...');
            const { idToken, decodedIdToken, refreshToken, customToken } = await handleTokenRefresh(tokens.refreshToken, {
                referer,
                enableCustomToken: options.enableCustomToken
            });
            (0, index_js_2.debug)('Token refreshed successfully. Updating response cookie headers...');
            const tokensToSign = {
                idToken,
                refreshToken,
                customToken
            };
            const cookies = new AuthCookies_js_1.AuthCookies(RequestCookiesProvider_js_1.RequestCookiesProvider.fromHeaders(request.headers), options);
            await cookies.setAuthCookies(tokensToSign, request.cookies);
            (0, verification_js_1.markCookiesAsVerified)(request.cookies);
            const response = await handleValidToken({ token: idToken, decodedToken: decodedIdToken, customToken }, request.headers);
            (0, index_js_2.debug)('Successfully handled authenticated response');
            validateResponse(response);
            await cookies.setAuthHeaders(tokensToSign, response.headers);
            return response;
        }, async (e) => {
            if (e instanceof error_js_1.AuthError &&
                e.code === error_js_1.AuthErrorCode.NO_MATCHING_KID) {
                throw error_js_1.InvalidTokenError.fromError(e, error_js_1.InvalidTokenReason.INVALID_KID);
            }
            (0, index_js_2.debug)('Authentication failed with error', { error: e });
            return handleError(e);
        }, options.experimental_enableTokenRefreshOnExpiredKidHeader ?? false);
    }
    catch (error) {
        if ((0, error_js_1.isInvalidTokenError)(error)) {
            (0, index_js_2.debug)(`Token is missing or has incorrect formatting. This is expected and usually means that user has not yet logged in`, {
                reason: error.reason
            });
            if (options.experimental_createAnonymousUserIfUserNotFound) {
                const { idToken, refreshToken } = await createAnonymousUser(options.apiKey);
                const tokensToSign = {
                    idToken,
                    refreshToken
                };
                const cookies = new AuthCookies_js_1.AuthCookies(RequestCookiesProvider_js_1.RequestCookiesProvider.fromHeaders(request.headers), options);
                await cookies.setAuthCookies(tokensToSign, request.cookies);
                const decodedToken = await verifyIdToken(idToken, {
                    checkRevoked: options.checkRevoked,
                    referer
                });
                (0, verification_js_1.markCookiesAsVerified)(request.cookies);
                const response = await handleValidToken({ token: idToken, decodedToken }, request.headers);
                validateResponse(response);
                await cookies.setAuthHeaders(tokensToSign, response.headers);
                return response;
            }
            return handleInvalidToken(error.reason);
        }
        throw error;
    }
}
