"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.validateOptions = validateOptions;
exports.getRequestCookiesTokens = getRequestCookiesTokens;
exports.getTokens = getTokens;
exports.getCookiesTokens = getCookiesTokens;
exports.getApiRequestTokens = getApiRequestTokens;
exports.getTokensFromObject = getTokensFromObject;
const jose_1 = require("jose");
const error_js_1 = require("../auth/error.js");
const index_js_1 = require("../auth/index.js");
const utils_js_1 = require("../auth/utils.js");
const index_js_2 = require("../debug/index.js");
const AuthCookies_js_1 = require("./cookies/AuthCookies.js");
const CookieParserFactory_js_1 = require("./cookies/parser/CookieParserFactory.js");
const RequestCookiesProvider_js_1 = require("./cookies/parser/RequestCookiesProvider.js");
const verification_js_1 = require("./cookies/verification.js");
const utils_js_2 = require("./utils.js");
function validateOptions(options) {
    if (!options.cookieSignatureKeys.length || !options.cookieSignatureKeys[0]) {
        throw new Error(`Expected cookieSignatureKeys to contain at least one signature key. Received: ${JSON.stringify(options.cookieSignatureKeys)}`);
    }
}
function getRequestCookiesTokens(cookies, options) {
    const parser = CookieParserFactory_js_1.CookieParserFactory.fromRequestCookies(cookies, options);
    return parser.parseCookies();
}
function toTokens(result) {
    if (!result) {
        return null;
    }
    return {
        token: result.idToken,
        decodedToken: result.decodedIdToken,
        customToken: result.customToken
    };
}
function isReadonly(cookies) {
    return !Object.hasOwn(cookies, 'set');
}
async function getTokens(cookies, options) {
    const now = Date.now();
    if (options.debug) {
        (0, index_js_2.enableDebugMode)();
    }
    validateOptions(options);
    const referer = options.headers ? (0, utils_js_2.getReferer)(options.headers) : '';
    const { verifyAndRefreshExpiredIdToken } = (0, index_js_1.getFirebaseAuth)({
        serviceAccount: options.serviceAccount,
        apiKey: options.apiKey,
        tenantId: options.tenantId
    });
    try {
        const tokens = await getRequestCookiesTokens(cookies, options);
        (0, index_js_2.debug)('getTokens: Tokens successfully extracted from cookies');
        if ((0, verification_js_1.areCookiesVerifiedByMiddleware)(cookies)) {
            (0, index_js_2.debug)('getTokens: Cookies are marked as verified. Skipping verification');
            const payload = (0, jose_1.decodeJwt)(tokens.idToken);
            return {
                token: tokens.idToken,
                decodedToken: (0, utils_js_1.mapJwtPayloadToDecodedIdToken)(payload),
                customToken: tokens.customToken
            };
        }
        const result = await verifyAndRefreshExpiredIdToken(tokens, {
            referer,
            experimental_enableTokenRefreshOnExpiredKidHeader: options.experimental_enableTokenRefreshOnExpiredKidHeader,
            async onTokenRefresh({ idToken, refreshToken, customToken }) {
                const cookieSerializeOptions = options.cookieSerializeOptions;
                if (isReadonly(cookies) || !cookieSerializeOptions) {
                    (0, index_js_2.debug)('getTokens: Expired tokens have been refreshed, but were not set on the response', {
                        hasCookieSerializeOptions: !!cookieSerializeOptions,
                        isReadonly: isReadonly(cookies)
                    });
                    return;
                }
                (0, index_js_2.debug)('getTokens: Expired tokens have been refreshed and new credentials will be set on the response');
                const tokensToSign = {
                    idToken,
                    refreshToken,
                    customToken
                };
                const setAuthCookiesOptions = {
                    ...options,
                    cookieSerializeOptions
                };
                const authCookies = new AuthCookies_js_1.AuthCookies(RequestCookiesProvider_js_1.RequestCookiesProvider.fromRequestCookies(cookies), setAuthCookiesOptions);
                await authCookies.setAuthCookies(tokensToSign, cookies);
            }
        });
        return toTokens(result);
    }
    catch (error) {
        if ((0, error_js_1.isInvalidTokenError)(error)) {
            (0, index_js_2.debug)(`Token is missing or has incorrect formatting. This is expected and usually means that user has not yet logged in`, {
                reason: error.reason
            });
            return null;
        }
        throw error;
    }
    finally {
        (0, index_js_2.debug)(`getTokens: took ${(Date.now() - now) / 1000}ms`);
    }
}
function getCookiesTokens(cookies, options) {
    const parser = CookieParserFactory_js_1.CookieParserFactory.fromObject(cookies, options);
    return parser.parseCookies();
}
async function getApiRequestTokens(request, options) {
    const referer = request.headers.referer ?? '';
    const { verifyAndRefreshExpiredIdToken } = (0, index_js_1.getFirebaseAuth)({
        serviceAccount: options.serviceAccount,
        apiKey: options.apiKey,
        tenantId: options.tenantId
    });
    try {
        const tokens = await getCookiesTokens(request.cookies, options);
        if ((0, verification_js_1.isCookiesObjectVerifiedByMiddleware)(request.cookies)) {
            const payload = (0, jose_1.decodeJwt)(tokens.idToken);
            return {
                token: tokens.idToken,
                decodedToken: (0, utils_js_1.mapJwtPayloadToDecodedIdToken)(payload),
                customToken: tokens.customToken
            };
        }
        return toTokens(await verifyAndRefreshExpiredIdToken(tokens, { referer }));
    }
    catch (error) {
        if ((0, error_js_1.isInvalidTokenError)(error)) {
            return null;
        }
        throw error;
    }
}
async function getTokensFromObject(cookies, options) {
    const { verifyAndRefreshExpiredIdToken } = (0, index_js_1.getFirebaseAuth)({
        serviceAccount: options.serviceAccount,
        apiKey: options.apiKey,
        tenantId: options.tenantId
    });
    try {
        const tokens = await getCookiesTokens(cookies, options);
        if ((0, verification_js_1.isCookiesObjectVerifiedByMiddleware)(cookies)) {
            const payload = (0, jose_1.decodeJwt)(tokens.idToken);
            return {
                token: tokens.idToken,
                decodedToken: (0, utils_js_1.mapJwtPayloadToDecodedIdToken)(payload),
                customToken: tokens.customToken
            };
        }
        return toTokens(await verifyAndRefreshExpiredIdToken(tokens, {
            referer: ''
        }));
    }
    catch (error) {
        if ((0, error_js_1.isInvalidTokenError)(error)) {
            return null;
        }
        throw error;
    }
}
